package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5Account;
import com.ibiz5.app.core.sample.filter.IBZ5AccountSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5AccountService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5AccountMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[账户] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5AccountServiceImpl")
public class IBZ5AccountServiceImpl extends ServiceImpl<IBZ5AccountMapper, IBZ5Account> implements IIBZ5AccountService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5Account et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5accountid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5Account> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5Account et) {
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5accountid",et.getIbz5accountid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5accountid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5Account> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5Account get(String key) {
        IBZ5Account et = getById(key);
        if(et==null){
            et=new IBZ5Account();
            et.setIbz5accountid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5Account getDraft(IBZ5Account et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5Account et) {
        return (!ObjectUtils.isEmpty(et.getIbz5accountid()))&&(!Objects.isNull(this.getById(et.getIbz5accountid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5Account et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5Account et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5Account> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5Account> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5Account> searchDefault(IBZ5AccountSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Account> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Account>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5Account> getIbz5accountByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5Account> getIbz5accountByEntities(List<IBZ5Account> entities) {
        List ids =new ArrayList();
        for(IBZ5Account entity : entities){
            Serializable id=entity.getIbz5accountid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



