package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5CPU;
import com.ibiz5.app.core.sample.filter.IBZ5CPUSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5CPUService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5CPUMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[处理器] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5CPUServiceImpl")
public class IBZ5CPUServiceImpl extends ServiceImpl<IBZ5CPUMapper, IBZ5CPU> implements IIBZ5CPUService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5CPU et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5cpuid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5CPU> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5CPU et) {
        ibz5hardwareService.update(ibz5cpuInheritMapping.toIbz5hardware(et));
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5cpuid",et.getIbz5cpuid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5cpuid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5CPU> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibz5hardwareService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5CPU get(String key) {
        IBZ5CPU et = getById(key);
        if(et==null){
            et=new IBZ5CPU();
            et.setIbz5cpuid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5CPU getDraft(IBZ5CPU et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5CPU et) {
        return (!ObjectUtils.isEmpty(et.getIbz5cpuid()))&&(!Objects.isNull(this.getById(et.getIbz5cpuid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5CPU et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5CPU et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5CPU> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5CPU> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5CPU> searchDefault(IBZ5CPUSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5CPU> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5CPU>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    com.ibiz5.app.core.sample.mapping.IBZ5CPUInheritMapping ibz5cpuInheritMapping;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5HardwareService ibz5hardwareService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(IBZ5CPU et){
        if(ObjectUtils.isEmpty(et.getIbz5cpuid()))
            et.setIbz5cpuid((String)et.getDefaultKey(true));
        com.ibiz5.app.core.sample.domain.IBZ5Hardware ibz5hardware =ibz5cpuInheritMapping.toIbz5hardware(et);
        ibz5hardware.set("ibz5hardwaretype","CPU");
        ibz5hardwareService.create(ibz5hardware);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5CPU> getIbz5cpuByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5CPU> getIbz5cpuByEntities(List<IBZ5CPU> entities) {
        List ids =new ArrayList();
        for(IBZ5CPU entity : entities){
            Serializable id=entity.getIbz5cpuid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



