package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5Hardware;
import com.ibiz5.app.core.sample.filter.IBZ5HardwareSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5HardwareService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5HardwareMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[硬件] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5HardwareServiceImpl")
public class IBZ5HardwareServiceImpl extends ServiceImpl<IBZ5HardwareMapper, IBZ5Hardware> implements IIBZ5HardwareService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5Hardware et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5hardwareid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5Hardware> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5Hardware et) {
        ibz5uniproductService.update(ibz5hardwareInheritMapping.toIbz5uniproduct(et));
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5hardwareid",et.getIbz5hardwareid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5hardwareid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5Hardware> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibz5uniproductService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5Hardware get(String key) {
        IBZ5Hardware et = getById(key);
        if(et==null){
            et=new IBZ5Hardware();
            et.setIbz5hardwareid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5Hardware getDraft(IBZ5Hardware et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5Hardware et) {
        return (!ObjectUtils.isEmpty(et.getIbz5hardwareid()))&&(!Objects.isNull(this.getById(et.getIbz5hardwareid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5Hardware et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5Hardware et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5Hardware> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5Hardware> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5Hardware> searchDefault(IBZ5HardwareSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Hardware> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Hardware>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 IndexDER
     */
    @Override
    public Page<IBZ5Hardware> searchIndexDER(IBZ5HardwareSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Hardware> pages=baseMapper.searchIndexDER(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Hardware>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    com.ibiz5.app.core.sample.mapping.IBZ5HardwareInheritMapping ibz5hardwareInheritMapping;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5UNIProductService ibz5uniproductService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(IBZ5Hardware et){
        if(ObjectUtils.isEmpty(et.getIbz5hardwareid()))
            et.setIbz5hardwareid((String)et.getDefaultKey(true));
        com.ibiz5.app.core.sample.domain.IBZ5UNIProduct ibz5uniproduct =ibz5hardwareInheritMapping.toIbz5uniproduct(et);
        ibz5uniproduct.set("ibz5uniproducttype","Hardware");
        ibz5uniproductService.create(ibz5uniproduct);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5Hardware> getIbz5hardwareByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5Hardware> getIbz5hardwareByEntities(List<IBZ5Hardware> entities) {
        List ids =new ArrayList();
        for(IBZ5Hardware entity : entities){
            Serializable id=entity.getIbz5hardwareid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



