package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5OrderDetail;
import com.ibiz5.app.core.sample.filter.IBZ5OrderDetailSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5OrderDetailService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5OrderDetailMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[订单明细] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5OrderDetailServiceImpl")
public class IBZ5OrderDetailServiceImpl extends ServiceImpl<IBZ5OrderDetailMapper, IBZ5OrderDetail> implements IIBZ5OrderDetailService {

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5OrderService ibz5orderService;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5UNIProductService ibz5uniproductService;

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.logic.IIBZ5OrderDetailCountMoneyLogic countmoneyLogic;

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.logic.IIBZ5OrderDetailGetUnitAndPriceLogic getunitandpriceLogic;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5OrderDetail et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5orderdetailid()),et);
        updateParentData(et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5OrderDetail> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
        updateParentDataBatch(list);
    }

    @Override
    @Transactional
    public IBZ5OrderDetail createTemp(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail createTempMajor(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean update(IBZ5OrderDetail et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5orderdetailid",et.getIbz5orderdetailid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5orderdetailid()),et);
        updateParentData(et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5OrderDetail> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
        updateParentDataBatch(list);
    }

    @Override
    @Transactional
    public IBZ5OrderDetail updateTemp(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail updateTempMajor(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        IBZ5OrderDetail entity=this.get(key);
        boolean result=removeById(key);
        updateParentData(entity);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        List<IBZ5OrderDetail> entities= baseMapper.selectBatchIds(idList);
        removeByIds(idList);
        updateParentDataBatch(entities);
    }

    @Override
    @Transactional
    public IBZ5OrderDetail removeTemp(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail removeTempMajor(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail get(String key) {
        IBZ5OrderDetail et = getById(key);
        if(et==null){
            et=new IBZ5OrderDetail();
            et.setIbz5orderdetailid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail getTemp(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail getTempMajor(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    public IBZ5OrderDetail getDraft(IBZ5OrderDetail et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail getDraftTemp(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail getDraftTempMajor(IBZ5OrderDetail et) {
        //自定义代码
        return et;
    }

    @Override
    public boolean checkKey(IBZ5OrderDetail et) {
        return (!ObjectUtils.isEmpty(et.getIbz5orderdetailid()))&&(!Objects.isNull(this.getById(et.getIbz5orderdetailid())));
    }
    @Override
    @Transactional
    public IBZ5OrderDetail countMoney(IBZ5OrderDetail et) {
        countmoneyLogic.execute(et);
         return et ;
    }

    @Override
    @Transactional
    public IBZ5OrderDetail getUnitAndPrice(IBZ5OrderDetail et) {
        getunitandpriceLogic.execute(et);
         return et ;
    }

    @Override
    @Transactional
    public boolean save(IBZ5OrderDetail et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5OrderDetail et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5OrderDetail> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5OrderDetail> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<IBZ5OrderDetail> selectByIbz5orderid(String ibz5orderid) {
        return baseMapper.selectByIbz5orderid(ibz5orderid);
    }
    @Override
    public List<IBZ5OrderDetail> selectByIbz5orderid(Collection<String> ids) {
        return this.list(new QueryWrapper<IBZ5OrderDetail>().in("ibz5orderid",ids));
    }

    @Override
    public void removeByIbz5orderid(String ibz5orderid) {
        this.remove(new QueryWrapper<IBZ5OrderDetail>().eq("ibz5orderid",ibz5orderid));
    }

	@Override
    public List<IBZ5OrderDetail> selectByIbz5uniproductid(String ibz5uniproductid) {
        return baseMapper.selectByIbz5uniproductid(ibz5uniproductid);
    }
    @Override
    public List<IBZ5OrderDetail> selectByIbz5uniproductid(Collection<String> ids) {
        return this.list(new QueryWrapper<IBZ5OrderDetail>().in("ibz5uniproductid",ids));
    }

    @Override
    public void removeByIbz5uniproductid(String ibz5uniproductid) {
        this.remove(new QueryWrapper<IBZ5OrderDetail>().eq("ibz5uniproductid",ibz5uniproductid));
    }


    /**
     * 查询集合 测试明细打印
     */
    @Override
    public Page<IBZ5OrderDetail> searchCSMXDY(IBZ5OrderDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5OrderDetail> pages=baseMapper.searchCSMXDY(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5OrderDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5OrderDetail> searchDefault(IBZ5OrderDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5OrderDetail> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5OrderDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 查询订单明细中产品数量
     */
    @Override
    public Page<IBZ5OrderDetail> searchLOGIC_0007(IBZ5OrderDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5OrderDetail> pages=baseMapper.searchLOGIC_0007(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5OrderDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 x
     */
    @Override
    public Page<IBZ5OrderDetail> searchX(IBZ5OrderDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5OrderDetail> pages=baseMapper.searchX(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5OrderDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(IBZ5OrderDetail et){
        //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5ORDER_IBZ5ORDERID]
        if(!ObjectUtils.isEmpty(et.getIbz5orderid())){
            com.ibiz5.app.core.sample.domain.IBZ5Order ibz5order=et.getIbz5order();
            if(ObjectUtils.isEmpty(ibz5order)){
                com.ibiz5.app.core.sample.domain.IBZ5Order majorEntity=ibz5orderService.get(et.getIbz5orderid());
                et.setIbz5order(majorEntity);
                ibz5order=majorEntity;
            }
            et.setIbz5ordername(ibz5order.getIbz5ordername());
            et.setOrderuid(ibz5order.getOrderuid());
        }
        //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5UNIPRODUCT_IBZ5UNIPRODUCTID]
        if(!ObjectUtils.isEmpty(et.getIbz5uniproductid())){
            com.ibiz5.app.core.sample.domain.IBZ5UNIProduct ibz5uniproduct=et.getIbz5uniproduct();
            if(ObjectUtils.isEmpty(ibz5uniproduct)){
                com.ibiz5.app.core.sample.domain.IBZ5UNIProduct majorEntity=ibz5uniproductService.get(et.getIbz5uniproductid());
                et.setIbz5uniproduct(majorEntity);
                ibz5uniproduct=majorEntity;
            }
            et.setIbz5uniproductname(ibz5uniproduct.getIbz5uniproductname());
            et.setUnitprice(ibz5uniproduct.getUnitprice());
            et.setUnit(ibz5uniproduct.getUnit());
        }
    }

    /**
     * 更新父数据（实体关系属性映射）
     * @param et
     */
    private void updateParentData(IBZ5OrderDetail et){
        //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5ORDER_IBZ5ORDERID]
        if(!ObjectUtils.isEmpty(et.getIbz5orderid())){
            com.ibiz5.app.core.sample.domain.IBZ5Order ibz5order=new com.ibiz5.app.core.sample.domain.IBZ5Order();
            List<Object> objects_0 = baseMapper.selectObjs(new QueryWrapper().select("sum(amount)").eq(true,"ibz5orderid",et.getIbz5orderid()));
            if(!ObjectUtils.isEmpty(objects_0) && objects_0.size()>0){
                ibz5order.set("amount",objects_0.get(0));
            }
            ibz5order.set("detailnum",baseMapper.selectCount(new QueryWrapper().eq(true,"ibz5orderid",et.getIbz5orderid())));
            ibz5order.set("ibz5orderid",et.getIbz5orderid());
            ibz5orderService.update(ibz5order);
        }
        //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5UNIPRODUCT_IBZ5UNIPRODUCTID]
        if(!ObjectUtils.isEmpty(et.getIbz5uniproductid())){
        }
    }

    /**
     * 批量更新父数据（实体关系属性映射）
     * @param entities
     */
    private void updateParentDataBatch(List <IBZ5OrderDetail> entities){
        Map <Object,IBZ5OrderDetail> map =new HashMap<>();
        if(entities.size()>0){
            for(IBZ5OrderDetail entity: entities){
                 //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5ORDER_IBZ5ORDERID]
                 if(!ObjectUtils.isEmpty(entity.getIbz5orderid()) && !map.containsKey(entity.getIbz5orderid())){
                    map.put(entity.getIbz5orderid(),entity);
                 }
                 //实体关系[DER1N_IBZ5ORDERDETAIL_IBZ5UNIPRODUCT_IBZ5UNIPRODUCTID]
                 if(!ObjectUtils.isEmpty(entity.getIbz5uniproductid()) && !map.containsKey(entity.getIbz5uniproductid())){
                    map.put(entity.getIbz5uniproductid(),entity);
                 }
            }
            if(map.size()>0){
                map.values().forEach(entity -> updateParentData(entity));
            }
        }
    }


    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5OrderDetail> getIbz5orderdetailByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5OrderDetail> getIbz5orderdetailByEntities(List<IBZ5OrderDetail> entities) {
        List ids =new ArrayList();
        for(IBZ5OrderDetail entity : entities){
            Serializable id=entity.getIbz5orderdetailid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



