package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5Order;
import com.ibiz5.app.core.sample.filter.IBZ5OrderSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5OrderService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5OrderMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[订单] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5OrderServiceImpl")
public class IBZ5OrderServiceImpl extends ServiceImpl<IBZ5OrderMapper, IBZ5Order> implements IIBZ5OrderService {

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5OrderDetailService ibz5orderdetailService;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5CustomerService ibz5customerService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5Order et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5orderid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5Order> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public IBZ5Order createTemp(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order createTempMajor(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean update(IBZ5Order et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5orderid",et.getIbz5orderid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5orderid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5Order> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public IBZ5Order updateTemp(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order updateTempMajor(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!ObjectUtils.isEmpty(ibz5orderdetailService.selectByIbz5orderid(key)))
            throw new BadRequestAlertException("删除数据失败，当前数据存在关系实体[IBZ5OrderDetail]数据，无法删除!","","");
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        if(!ObjectUtils.isEmpty(ibz5orderdetailService.selectByIbz5orderid(idList)))
            throw new BadRequestAlertException("删除数据失败，当前数据存在关系实体[IBZ5OrderDetail]数据，无法删除!","","");
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5Order removeTemp(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order removeTempMajor(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order get(String key) {
        IBZ5Order et = getById(key);
        if(et==null){
            et=new IBZ5Order();
            et.setIbz5orderid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order getTemp(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order getTempMajor(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    public IBZ5Order getDraft(IBZ5Order et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order getDraftTemp(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order getDraftTempMajor(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public IBZ5Order bUIAction(IBZ5Order et) {
        //自定义代码
        return et;
    }

    @Override
    public boolean checkKey(IBZ5Order et) {
        return (!ObjectUtils.isEmpty(et.getIbz5orderid()))&&(!Objects.isNull(this.getById(et.getIbz5orderid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5Order et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5Order et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5Order> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5Order> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<IBZ5Order> selectByIbz5customerid(String ibz5customerid) {
        return baseMapper.selectByIbz5customerid(ibz5customerid);
    }
    @Override
    public void removeByIbz5customerid(String ibz5customerid) {
        this.remove(new QueryWrapper<IBZ5Order>().eq("ibz5customerid",ibz5customerid));
    }


    /**
     * 查询集合 测试打印
     */
    @Override
    public Page<IBZ5Order> searchCSDY(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchCSDY(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 当前用户已审核
     */
    @Override
    public Page<IBZ5Order> searchCurState30(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchCurState30(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 订单金额统计报表
     */
    @Override
    public Page<IBZ5Order> searchDDJETJBB(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchDDJETJBB(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 订单报表打印
     */
    @Override
    public Page<IBZ5Order> searchDDReport(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchDDReport(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 打印订单明细数据源
     */
    @Override
    public Page<IBZ5Order> searchDYDDMXSJY(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchDYDDMXSJY(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5Order> searchDefault(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 图表默认数据源
     */
    @Override
    public Page<IBZ5Order> searchEchartsDefaultDataSource(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchEchartsDefaultDataSource(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 仪表盘数据源
     */
    @Override
    public Page<IBZ5Order> searchGaugeDataSource(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchGaugeDataSource(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 仪表数据源（时间）
     */
    @Override
    public Page<IBZ5Order> searchGaugeDataSourceByTime(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchGaugeDataSourceByTime(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 当前用户审核通过与全部取消
     */
    @Override
    public Page<IBZ5Order> searchLOGIC_22(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchLOGIC_22(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 雷达图数据源
     */
    @Override
    public Page<IBZ5Order> searchRadarDataSource(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchRadarDataSource(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 审核中订单
     */
    @Override
    public Page<IBZ5Order> searchSHZOrder(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchSHZOrder(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 订单状态10
     */
    @Override
    public Page<IBZ5Order> searchState10(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchState10(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 订单状态30
     */
    @Override
    public Page<IBZ5Order> searchState30(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchState30(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 订单状态40
     */
    @Override
    public Page<IBZ5Order> searchState40(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchState40(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 小额查询
     */
    @Override
    public Page<IBZ5Order> searchXECX(IBZ5OrderSearchContext context) {
        fillWFTaskContext(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Order> pages=baseMapper.searchXECX(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Order>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Autowired
    private com.ibiz5.app.util.client.IBZWFFeignClient ibzwfFeignClient;

    /**
     * 查询工作流待办
     * @param context
     */
    private void fillWFTaskContext(IBZ5OrderSearchContext context){

        if(!StringUtils.isEmpty(context.getUserTaskId()) && !StringUtils.isEmpty(context.getProcessDefinitionKey())){
            List<String> businessKeys= ibzwfFeignClient.getbusinesskeysByUserId("demosys", com.ibiz5.app.util.security.AuthenticationUser.getAuthenticationUser().getUserid(),"ibz5orders",context.getProcessDefinitionKey(),context.getUserTaskId());
            if(businessKeys.size()>0){
                context.getSelectCond().in("ibz5orderid",businessKeys);
            }
            else{
                context.getSelectCond().apply("1<>1");
            }
        }
    }

    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(IBZ5Order et){
        //实体关系[DER1N_IBZ5ORDER_IBZ5CUSTOMER_IBZ5CUSTOMERID]
        if(!ObjectUtils.isEmpty(et.getIbz5customerid())){
            com.ibiz5.app.core.sample.domain.IBZ5Customer ibz5customer=et.getIbz5customer();
            if(ObjectUtils.isEmpty(ibz5customer)){
                com.ibiz5.app.core.sample.domain.IBZ5Customer majorEntity=ibz5customerService.get(et.getIbz5customerid());
                et.setIbz5customer(majorEntity);
                ibz5customer=majorEntity;
            }
            et.setIbz5customername(ibz5customer.getIbz5customername());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5Order> getIbz5orderByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5Order> getIbz5orderByEntities(List<IBZ5Order> entities) {
        List ids =new ArrayList();
        for(IBZ5Order entity : entities){
            Serializable id=entity.getIbz5orderid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

    @Autowired
    @Lazy
    IIBZ5OrderService proxyService;

    @Value("${ibiz.syncImportLimit:1000}")
    private int syncImportLimit;

    /**
     * 上传数据检查
     * @param entities
     * @param isIgnoreError
     * @return
     */
    private JSONObject testImportData(List<IBZ5Order> entities,boolean isIgnoreError) {

        JSONObject rs=new JSONObject();
        Set ids=new HashSet<>();
        List<String> errorMsgs = new ArrayList<>();
        List<Integer> errorLines = new ArrayList<>();
        List<IBZ5Order> duplicateKeys=new ArrayList<>();
        String keyField= DEFieldCacheMap.getDEKeyField(IBZ5Order.class);
        if(ObjectUtils.isEmpty(keyField)){
            errorLines.add(1);
            rs.put("rst", 1);
            rs.put("msg", "数据导入失败，未能获取到实体[IBZ5Order]的主键属性");
            rs.put("errorLines", errorLines);
            return rs;
        }
        //主键重复性判断.外键约束判断（上传数据自身的检查/数据库的检查）
        for(int i=0;i<entities.size();i++) {
            IBZ5Order entity = entities.get(i);
            Object id = entity.getIbz5orderid();
            if(ObjectUtils.isEmpty(id)) {
                id = entity.getDefaultKey(true);
                if(ObjectUtils.isEmpty(id)){
                    Integer lineNum = i + 1;
                    errorLines.add(lineNum);
                    errorMsgs.add("第" + lineNum + "行：无法获取当前数据主键。");
                    continue;
                }
                else{
                    entity.setIbz5orderid((String) id);
                }
            }
            if(!ids.contains(id)){
                ids.add(id);
            }
            else{
                Integer lineNum = i + 1;
                errorLines.add(lineNum);
                errorMsgs.add("第" + lineNum + "行：导入数据之间存在重复数据。");
                if(isIgnoreError){
                    duplicateKeys.add(entity);
                    continue;
                }
                else{
                    break;
                }
            }
        //实体关系[DER1N_IBZ5ORDER_IBZ5CUSTOMER_IBZ5CUSTOMERID]
        if(!ObjectUtils.isEmpty(entity.getIbz5customerid())){
            com.ibiz5.app.core.sample.domain.IBZ5Customer fkEntity=new com.ibiz5.app.core.sample.domain.IBZ5Customer();
            fkEntity.setIbz5customerid(entity.getIbz5customerid());
            if(!ibz5customerService.checkKey(fkEntity)){
                Integer lineNum = i + 1;
                errorLines.add(lineNum);
                errorMsgs.add(String.format("第" + lineNum + "行：[%s]父数据有误。",entity.getIbz5customerid()));
                if(isIgnoreError){
                    entity.setIbz5customerid(null);
                    continue;
                }
                else{
                   break;
                }
            }
        }
        }
        if(duplicateKeys.size()>0){
            for(IBZ5Order duplicateKey:duplicateKeys){
                entities.remove(duplicateKey);
            }
        }
        if (errorMsgs.size() > 0) {
            rs.put("rst", 1);
            rs.put("msg", String.join("<br>", errorMsgs));
            rs.put("errorLines", errorLines);
            return rs;
        }
        rs.put("rst", 0);
        return rs;
    }

    /**
     * 实体数据导入
     * @param entities
     * @param batchSize
     * @param isIgnoreError
     * @return
     */
    @Override
    @Transactional
    public JSONObject importData(List<IBZ5Order> entities, int batchSize ,boolean isIgnoreError) {
        if(entities.size()>syncImportLimit){
            proxyService.asyncImportData(entities,batchSize,isIgnoreError);
            JSONObject rs=new JSONObject();
            rs.put("rst", 0);
            rs.put("msg",String.format("当前导入数据已超过同步导入数量上限[%s]，系统正在进行异步导入，请稍后!",syncImportLimit));
            rs.put("data",entities);
            return rs;
        }
        else{
            return syncImportData(entities,batchSize,isIgnoreError);
        }
    }

    @Transactional
    public void asyncImportData(List<IBZ5Order> entities, int batchSize ,boolean isIgnoreError){
        executeImportData(entities,batchSize,isIgnoreError);
    }

    @Transactional
    public JSONObject syncImportData(List<IBZ5Order> entities, int batchSize ,boolean isIgnoreError){
        return executeImportData(entities,batchSize,isIgnoreError);
    }

    @Transactional
    public JSONObject executeImportData(List<IBZ5Order> entities, int batchSize ,boolean isIgnoreError) {
        JSONObject rs=testImportData(entities,isIgnoreError);
        if(rs.getInteger("rst")==1 && !isIgnoreError)
            return rs;
        List<IBZ5Order> tempDEList=new ArrayList<>();
        Set tempIds=new HashSet<>();

        for(int i=0;i<entities.size();i++) {
            IBZ5Order entity = entities.get(i);
            tempDEList.add(entity);
            Object id=entity.getIbz5orderid();
            if(!ObjectUtils.isEmpty(id))
                tempIds.add(id);
            if(tempDEList.size()>=batchSize || (tempDEList.size()<batchSize && i==entities.size()-1)){
                commit(tempDEList,tempIds);
                tempDEList.clear();
                tempIds.clear();
                }
            }
        rs.put("rst", 0);
        rs.put("data",entities);
        return rs;
    }

    /**
     * 批量提交
     * @param entities 数据
     * @param ids 要提交数据的id
     */
    @Transactional
    public void commit(List<IBZ5Order> entities, Set ids){
        List<IBZ5Order> _create=new ArrayList<>();
        List<IBZ5Order> _update=new ArrayList<>();
        Set oldIds=new HashSet<>();
        if(ids.size()>0){
            List<IBZ5Order> oldEntities=this.listByIds(ids);
            for(IBZ5Order entity:oldEntities){
                oldIds.add(entity.getIbz5orderid());
            }
        }
        for(IBZ5Order entity:entities){
            Object id=entity.getIbz5orderid();
            if(oldIds.contains(id))
                _update.add(entity);
            else
                _create.add(entity);
        }
        if(_update.size()>0)
            proxyService.updateBatch(_update);
        if(_create.size()>0)
            proxyService.createBatch(_create);
    }


}



