package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5OrderType;
import com.ibiz5.app.core.sample.filter.IBZ5OrderTypeSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5OrderTypeService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5OrderTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[订单类型] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5OrderTypeServiceImpl")
public class IBZ5OrderTypeServiceImpl extends ServiceImpl<IBZ5OrderTypeMapper, IBZ5OrderType> implements IIBZ5OrderTypeService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5OrderType et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5ordertypeid()),et);
        com.ibiz5.app.core.sample.domain.IBZ5OrderType actionLogicDE =new com.ibiz5.app.core.sample.domain.IBZ5OrderType();
        et.copyTo(actionLogicDE,true);
        ibz5ordertypeService.resetCodeList(actionLogicDE);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5OrderType> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5OrderType et) {
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5ordertypeid",et.getIbz5ordertypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5ordertypeid()),et);
        com.ibiz5.app.core.sample.domain.IBZ5OrderType actionLogicDE =new com.ibiz5.app.core.sample.domain.IBZ5OrderType();
        et.copyTo(actionLogicDE,true);
        ibz5ordertypeService.resetCodeList(actionLogicDE);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5OrderType> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        IBZ5OrderType et=new IBZ5OrderType();
        et.set("ibz5ordertypeid",key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5OrderType get(String key) {
        IBZ5OrderType et = getById(key);
        if(et==null){
            et=new IBZ5OrderType();
            et.setIbz5ordertypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5OrderType getDraft(IBZ5OrderType et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5OrderType et) {
        return (!ObjectUtils.isEmpty(et.getIbz5ordertypeid()))&&(!Objects.isNull(this.getById(et.getIbz5ordertypeid())));
    }
    @Override
    @Transactional
    public IBZ5OrderType resetCodeList(IBZ5OrderType et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean save(IBZ5OrderType et) {
        if(!saveOrUpdate(et))
            return false;
        com.ibiz5.app.core.sample.domain.IBZ5OrderType actionLogicDE =new com.ibiz5.app.core.sample.domain.IBZ5OrderType();
        et.copyTo(actionLogicDE,true);
        ibz5ordertypeService.resetCodeList(actionLogicDE);
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5OrderType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5OrderType> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5OrderType> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5OrderType> searchDefault(IBZ5OrderTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5OrderType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5OrderType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5OrderType> getIbz5ordertypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5OrderType> getIbz5ordertypeByEntities(List<IBZ5OrderType> entities) {
        List ids =new ArrayList();
        for(IBZ5OrderType entity : entities){
            Serializable id=entity.getIbz5ordertypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



