package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5Sample0020;
import com.ibiz5.app.core.sample.filter.IBZ5Sample0020SearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5Sample0020Service;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5Sample0020Mapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[员工] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5Sample0020ServiceImpl")
public class IBZ5Sample0020ServiceImpl extends ServiceImpl<IBZ5Sample0020Mapper, IBZ5Sample0020> implements IIBZ5Sample0020Service {

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5Sample0019Service ibz5sample0019Service;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5Sample0020 et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5sample0020id()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5Sample0020> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5Sample0020 et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5sample0020id",et.getIbz5sample0020id())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5sample0020id()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5Sample0020> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5Sample0020 get(String key) {
        IBZ5Sample0020 et = getById(key);
        if(et==null){
            et=new IBZ5Sample0020();
            et.setIbz5sample0020id(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5Sample0020 getDraft(IBZ5Sample0020 et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(IBZ5Sample0020 et) {
        return (!ObjectUtils.isEmpty(et.getIbz5sample0020id()))&&(!Objects.isNull(this.getById(et.getIbz5sample0020id())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5Sample0020 et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5Sample0020 et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5Sample0020> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5Sample0020> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<IBZ5Sample0020> selectByIbz5sample0019id(String ibz5sample0019id) {
        return baseMapper.selectByIbz5sample0019id(ibz5sample0019id);
    }
    @Override
    public void removeByIbz5sample0019id(String ibz5sample0019id) {
        this.remove(new QueryWrapper<IBZ5Sample0020>().eq("ibz5sample0019id",ibz5sample0019id));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5Sample0020> searchDefault(IBZ5Sample0020SearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5Sample0020> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5Sample0020>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(IBZ5Sample0020 et){
        //实体关系[DER1N_IBZ5SAMPLE0020_IBZ5SAMPLE0019_IBZ5SAMPLE0019ID]
        if(!ObjectUtils.isEmpty(et.getIbz5sample0019id())){
            com.ibiz5.app.core.sample.domain.IBZ5Sample0019 ibz5sample0019=et.getIbz5sample0019();
            if(ObjectUtils.isEmpty(ibz5sample0019)){
                com.ibiz5.app.core.sample.domain.IBZ5Sample0019 majorEntity=ibz5sample0019Service.get(et.getIbz5sample0019id());
                et.setIbz5sample0019(majorEntity);
                ibz5sample0019=majorEntity;
            }
            et.setIbz5sample0019name(ibz5sample0019.getIbz5sample0019name());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5Sample0020> getIbz5sample0020ByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5Sample0020> getIbz5sample0020ByEntities(List<IBZ5Sample0020> entities) {
        List ids =new ArrayList();
        for(IBZ5Sample0020 entity : entities){
            Serializable id=entity.getIbz5sample0020id();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



