package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5SoftwareSuit;
import com.ibiz5.app.core.sample.filter.IBZ5SoftwareSuitSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5SoftwareSuitService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5SoftwareSuitMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[软件] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5SoftwareSuitServiceImpl")
public class IBZ5SoftwareSuitServiceImpl extends ServiceImpl<IBZ5SoftwareSuitMapper, IBZ5SoftwareSuit> implements IIBZ5SoftwareSuitService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5SoftwareSuit et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5softwaresuitid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5SoftwareSuit> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5SoftwareSuit et) {
        ibz5uniproductService.update(ibz5softwaresuitInheritMapping.toIbz5uniproduct(et));
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5softwaresuitid",et.getIbz5softwaresuitid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5softwaresuitid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5SoftwareSuit> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibz5uniproductService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5SoftwareSuit get(String key) {
        IBZ5SoftwareSuit et = getById(key);
        if(et==null){
            et=new IBZ5SoftwareSuit();
            et.setIbz5softwaresuitid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5SoftwareSuit getDraft(IBZ5SoftwareSuit et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5SoftwareSuit et) {
        return (!ObjectUtils.isEmpty(et.getIbz5softwaresuitid()))&&(!Objects.isNull(this.getById(et.getIbz5softwaresuitid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5SoftwareSuit et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5SoftwareSuit et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5SoftwareSuit> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5SoftwareSuit> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5SoftwareSuit> searchDefault(IBZ5SoftwareSuitSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5SoftwareSuit> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5SoftwareSuit>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    com.ibiz5.app.core.sample.mapping.IBZ5SoftwareSuitInheritMapping ibz5softwaresuitInheritMapping;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5UNIProductService ibz5uniproductService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(IBZ5SoftwareSuit et){
        if(ObjectUtils.isEmpty(et.getIbz5softwaresuitid()))
            et.setIbz5softwaresuitid((String)et.getDefaultKey(true));
        com.ibiz5.app.core.sample.domain.IBZ5UNIProduct ibz5uniproduct =ibz5softwaresuitInheritMapping.toIbz5uniproduct(et);
        ibz5uniproduct.set("ibz5uniproducttype","Software");
        ibz5uniproductService.create(ibz5uniproduct);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5SoftwareSuit> getIbz5softwaresuitByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5SoftwareSuit> getIbz5softwaresuitByEntities(List<IBZ5SoftwareSuit> entities) {
        List ids =new ArrayList();
        for(IBZ5SoftwareSuit entity : entities){
            Serializable id=entity.getIbz5softwaresuitid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



