package com.ibiz5.app.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import com.ibiz5.app.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import com.ibiz5.app.core.sample.domain.IBZ5UNIProduct;
import com.ibiz5.app.core.sample.filter.IBZ5UNIProductSearchContext;
import com.ibiz5.app.core.sample.service.IIBZ5UNIProductService;

import com.ibiz5.app.util.helper.CachedBeanCopier;
import com.ibiz5.app.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ibiz5.app.core.sample.mapper.IBZ5UNIProductMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[统一产品（索引主实体）] 服务对象接口实现
 */
@Slf4j
@Service("IBZ5UNIProductServiceImpl")
public class IBZ5UNIProductServiceImpl extends ServiceImpl<IBZ5UNIProductMapper, IBZ5UNIProduct> implements IIBZ5UNIProductService {

    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5OrderDetailService ibz5orderdetailService;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5Sample0011Service ibz5sample0011Service;
    @Autowired
    @Lazy
    protected com.ibiz5.app.core.sample.service.IIBZ5Sample0013Service ibz5sample0013Service;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBZ5UNIProduct et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5uniproductid()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBZ5UNIProduct> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBZ5UNIProduct et) {
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("ibz5uniproductid",et.getIbz5uniproductid())))
            return false;
        CachedBeanCopier.copy(get(et.getIbz5uniproductid()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBZ5UNIProduct> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!ObjectUtils.isEmpty(ibz5orderdetailService.selectByIbz5uniproductid(key)))
            throw new BadRequestAlertException("删除数据失败，当前数据存在关系实体[IBZ5OrderDetail]数据，无法删除!","","");
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        if(!ObjectUtils.isEmpty(ibz5orderdetailService.selectByIbz5uniproductid(idList)))
            throw new BadRequestAlertException("删除数据失败，当前数据存在关系实体[IBZ5OrderDetail]数据，无法删除!","","");
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBZ5UNIProduct get(String key) {
        IBZ5UNIProduct et = getById(key);
        if(et==null){
            et=new IBZ5UNIProduct();
            et.setIbz5uniproductid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public IBZ5UNIProduct getDraft(IBZ5UNIProduct et) {
        return et;
    }

    @Override
    public boolean checkKey(IBZ5UNIProduct et) {
        return (!ObjectUtils.isEmpty(et.getIbz5uniproductid()))&&(!Objects.isNull(this.getById(et.getIbz5uniproductid())));
    }
    @Override
    @Transactional
    public boolean save(IBZ5UNIProduct et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBZ5UNIProduct et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBZ5UNIProduct> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBZ5UNIProduct> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBZ5UNIProduct> searchDefault(IBZ5UNIProductSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5UNIProduct> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5UNIProduct>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 IndexDER
     */
    @Override
    public Page<IBZ5UNIProduct> searchIndexDER(IBZ5UNIProductSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBZ5UNIProduct> pages=baseMapper.searchIndexDER(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBZ5UNIProduct>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBZ5UNIProduct> getIbz5uniproductByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBZ5UNIProduct> getIbz5uniproductByEntities(List<IBZ5UNIProduct> entities) {
        List ids =new ArrayList();
        for(IBZ5UNIProduct entity : entities){
            Serializable id=entity.getIbz5uniproductid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



